;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; boot.asm - SimpleOS Bootloader Code
;; (c) 2001 Breckin Loggins
;; Copy freely
;;

;; I've tried to comment this code as much as possible.  I'll 
;; add more and more comments as it starts to work better.  My 
;; goal is for you to be able to read this bootsector and then
;; pretty much understand how to write one.  If there were any
;; GOTCHAS that I went through while writing it, I'll put those in
;; comments by where they occured so you won't make the same mistake
;; The comments in this code are VERY verbose, chatty actually, this
;; was intended.  I wanted this bootcode to read like an article

[BITS 16]	; Generate 16 bit code
org 0x0		; We want total control over memory translation

jmp short bootup	; We can use the area below for the data area

; The next instruction is padding to make it three bytes.  Now if you've
; read bootsector code before then you're probably saying 'OK, I've seen
; that before, but WHY THA HELL do I do it?'  Well, the answer is that
; if you're not using or need to have the disk accessible from the fat
; file system, then you don't need it.  Otherwise DOS/Other M$ OSes 
; expect a FAT info block 4 bytes into the first sector.  I obviously
; don't have one here, but in case I need to read this disk from my
; Win2K machine, all I have to do is past in that structure right below
; the NOP and WHAMMO!, instant FAT disk.  Just thought you'd like to know
NOP

; Data Area
; EQUS - They don't actually exist in memory
	STACKSEG EQU 0x8000	; Arbitrary location
	INITSEG  EQU 0x07c0	; Where we are right now
	VIDEOSEG EQU 0xb800	; The display screen
	NEWSEG	 EQU 0x9000	; Where we will put ourselves
	
	CODESEL  EQU 0x08	; Code selector
	DATASEL  EQU 0x10	; Data selector
	VIDEOSEL EQU 0x18	; Video selector
	
; VARIABLES - Actual memory locations
	
	; Bootdrive, we'll save where we booted from here
	bootdrv
		db 0
		
	; The global descriptor table register
	gdtr
		dw 0x1000        ; Size of the table 
		dd 0		 ; Base 0 (which are belong to us ;-)	
		
	; The interrupt descriptor table, nothing here yet
	idtr
		dw 0
		dd 0x1000
	
	; The global descriptor table
	gdt
		; ALWAYS set the first descriptor in the table
		; to zero.  Don't ask why, just do it
		dd 0			; Dummy descriptor
		dd 0
		
		; Code descriptor, 8 bytes in, so selector is (wait for it...)
		; 0x08
		dw 0xffff		; 4 GB limit
		dw 0x0			; Base zero
		db 0x0			; Base zero
		db 0x9a			; non-comforming exec segment
		db 0xcf			; Granularity 4kb
		db 0			; Base zero
		
		; Data descriptor, another 8 bytes in, so selector is
		; 0x10 (remember we're counting in hex here)
		dw 0xffff		; 4 GB limit
		dw 0x0			; Base zero
		db 0x0			; Base zero
		db 0x92			; Non conforming data seg
		db 0xcf			; Granularity 4 kb
		db 0
	gdt_end		; So we can dynamically calculate its length
	
	; Messages to be output to the screen.  All of these will
	; be output in 32 bit protected mode, and as such they do not
	; contain CR and/or LF because we don't have the code to handle
	; them (yet).
	
	a20_failed
		db 'A20 Check Failed - System Halted', 0
	
	
; PROCEDURES - Useful code
	
	; Nothing here for now, but note that there are some 
	; 32-bit procedures at the bottom of this file

; bootup code - this is where we jmped from above
bootup:
	mov [bootdrv], dl	; Save boot drive
	
	; Move us out of the way so we can feel free to maul lower memory
	cld			; Clear the direction flag
	mov ax, INITSEG
	mov ds, ax		; This is where we are now
	mov ax, NEWSEG
	mov es, ax		; This is where we're going
	xor di, di		; Clear offsets
	xor si, si		; Ditto
	mov cx, 0x200		; 512 Bytes
	
	cli			; No interrupts allowed
	rep movsb		; Do It!!
	
	; Ok we've moved it, now we have to jmp to where we've moved
	JMP NEWSEG:there
	
	there:
	;; And here we are, at 0x9000 instead of 0x07c0
	sti			; Turn interrupts back on
		
	;; Next we load the kernel starting at 0x10000
	
	; Reset the disk system
readme:
	mov ah, 0		; Reset function
				; DL contains drive num
	int 0x13		; Do it!
	
	; Load in at logical sector 2 and read in 10 sectors for now
	; thats 10*512 or 5K of code, should be ok for now
	cld			; Clear direction flag (I don't think 
				; it's neccessary)
	xor bx, bx		; Offset zero
	mov ax, 0x1000		; Segment 0x1000 (linear 0x10000)
	mov es, ax
	mov ax, 0x0210	        ; AH = 02 (function call), AL=16 (sectors)
	mov cx, 0x0002		; CH - first track, CL - logical sec 2
	mov dh, 0		; First head
	mov dl, [bootdrv]	; DL contains drive num
	int 0x13		; Do it!
	jc readme		; Retry on error, this might never end, 
				; but only if the disk is bad
				; This little trick stolen from the PolyOS
				; bootsector code
	
	; Set up data segment, extra segment, and video segment (gs)
	mov ax, STACKSEG
	mov ss, ax		; Stack segment is now 0x5000
	mov ax, 0xff		; Top of segment (only 256 bytes of
				; stackspace, but that's ok for now)
	mov sp, ax		; Stack pointer is now at top of segment
	
	mov ax, NEWSEG
	mov ds, ax		; Data seg is now 0x07c0
	xor si, si		; Offset 0 (top of this boot code)
	
	mov es, ax		; AX Still contains DATASEG
	xor bp, bp		; Offset 0
	
	mov ax, VIDEOSEG	
	mov gs, ax		; GS is now a pointer to vid mem
				; We'll use BP for the offset
	
	; Some previous code here checked SS:SP and GS 
	; and they were both operational, so now we have 
	; a stack and a place to write to video
	; HINT-HINT: This means once you get to this point
	; you should probably write some test code.  This means
	; pushing a value onto the new stack and popping it back
	; seeing if you get the same value, and also writing an
	; ASCII byte to an EVEN offset (muy importante) to GS to
	; see if something shows up on screen.
	
	; Some previous code here checked DS and ES
	; and they were both operational, so now we have
	; all of our segments (that we use) set up
	; As above, you should write some stuff here to write 
	; data to these segments, then get them back and see
	; if you get the same value
	
	; Disable interrupts and NMI
	; OK this needs a little explaining, interrupts are enabled and
	; disabled by certain flag states (look it up if you really want
	; to know).  Sometimes CLI doesn't do the job right, so we just do
	; it manually by pushinh a zero onto the stack and popping it into
	; eflags.  This has the nifty side effect of also making sure 
	; the TF isn't set or anything else that may have been toyed with 
	; by the (sometimes screwy) BIOS code that executed before us
	push dword 0
	popfd
	
	; This is code to disable NMI.  Look it up if you need to 
	; know how it works.  I found this code by doing a Google search
	; on 'disabling NMI'.  Usually the same article or site will
	; also tell you how to turn them back on
	in al, 0x70
	or al, 0x80
	out 0x70, al
	
	; Mov the gdt to linear zero. No, you don't have to do this, 
	; I did it initially to debug something that I thought was 
	; an invalid base address of the GDTR, but which turned out to
	; be an incorrect type bit in the table itself.  Some folks tell
	; you to put this at the bottom of linear address space.  This is 
	; nice, but really you can put it anywhere you want
	push es
	xor ax, ax
	mov es, ax
	xor di, di
	
	mov si, gdt
	mov cx, gdt_end-gdt
	rep movsb
	pop es
	
	; Now the gdt should be at linear zero
	
	lgdt [gdtr]		; Load the descriptor table!
	lidt [idtr]		; Load the interrupt table!
	
	; Enabling A20 gate - WARNING! This code assumes at least a
	;	PS/2 compatible system, if you have something older,  
	;	and you're looking at this code, then you probably 
	;	know how to fix it!
	in al, 0x92
	or al, 2
	out 0x92, al
	
	; Uh oh, here we go!
	mov eax, cr0
	inc eax			; Turn on pm bit, some people or al,1
				; but this is the way Linux does it, 
				; Nuff said.
	mov cr0, eax		; Do it!
	jmp short flush		; Flush the prefetch queue
	

flush:
	; Now we're actually IN protected mode, but all the segment
	; registers still contain their 16 bit values, so we need
	; to reset them.  Note you can still use the old values
	; UNTIL you issue the FAR JMP below.  After that you get
	; a protection fault
	
	; Set up segments
	mov bx, 0x10
	mov ds, bx
	mov es, bx
	mov fs, bx
	mov es, bx
	mov gs, bx
	mov ss, bx

	; We're ready to jmp!  You must convert go_pm (or whatever label you
	; use) to a 32-bit linear address, that means in this case 0x7cxx
	; or 0x90000 + label, you could precalc this elsewhere, but why?
	; This saves space for other more useful code
	; One other word on this: some bootloader code does this opcode
	; manually by defining the bytes that would make it up.  Well, 
	; if you're using NASM (and you should be :) ), you DON'T have to 
	; do that, the dword here tells NASM what you're doing and that 
	; yes, you KNOW you're jumping into 32-bit code from 16-bit code
	jmp dword 0x08:0x90000 + go_pm

go_pm:
[BITS 32]	
;; Do your 32-bit thing here, right now we'll just test the A20 line
;; and if the test passes we'll jump to our Kernel code

;; Check for A20.  If it's still not working, writing to 1 MB memory
;; location will simply result in the write "wrapping" around to the lower
;; 0MB memory location.  This won't hurt anything because if A20 is disabled
;; we'll halt the processor anyway

mov ebx, [0x1000]	  ; Get the memory value that exists here
mov eax, ebx		  ; Save it
not ebx			  ; Invert ebx
mov dword [0x101000], ebx ; Write the inverted value to high mem
mov ebx, [0x1000]	  ; Re-read the original location

;; eax and ebx should be equal, if not, A20 enable failed

cmp eax, ebx
je a20_enabled

;; If we got here, A20 failed.  Let the user know as such and then 
;; halt the processor

mov ebx, 0x9000 + a20_failed
mov ecx, 0xb8002
call putmsg32
hlt

a20_enabled:
;; We know A20 is on, we can get on with our lives

;; Reset EFlags
push dword 0
popfd

;; Jump to where we put our kernel, 0x10000

; Here's something that will REALLY throw you for a loop if you don't know 
; about it: you CAN NOT just JMP 0x10000.  I had no idea why this didn't
; work until I took a closer look at the 80386 reference manual, it seems
; that an immediate value is taken as a relative address, which means it 
; adds the specified amount to the current offset, which is probably not
; what you want.  Of course now it seems obvious, it allows you to do things
; like JMP $ which actually JMPs 0 bytes away.  To make it an absolute address
; you can do one of three things: put it in a register and then jmp to the 
; value in the register (like I did), do a far JMP to the current CODESEL, 
; or store the value in a memory address and then reference the memory.
mov eax, 0x10000		; Linear addy of the Kernel
jmp eax				; We're outta here!


;; This is the end of the road, if we get here something was wrong
;; (or I haven't finished writing the bootloader yet ;-) 
spin: 
	jmp spin	
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 32 - Bit Utility Functions
;;

;; FUNCTION: 
;;		putmsg32
;; USAGE: 
;;		ds assumed
;;		ebx contains linear message offset
;;		ecx contains linear address + offset into framebuf
;; REMARKS: 
;;		Oh dear, this in incredibly ugly, but hey it works
;;		I'll optimize/shrink it later.  Note that your
;;		string MUST contain a binary 0 byte at the end
;;		or Bad Things WILL happen.  This code also
;;		handles NULL strings, although I don't know why
;;		I can't imagine why someone would pass in a NULL
;;		string to a bootloader function, but it's there
putmsg32:
	push eax			; Save eax
	
	;; Ouch, I'm stupid.  Here's a lesson you can learn from me:
	;; when you loop to an instruction multiple times, it's best
	;; NOT to have pushes without accompanying pops in the loop code.
	;; I WAS just jumping to putmsg32, and that was pushing eax every time
	;; so of course when I issued a ret instruction it POPped back the
	;; value of eax and then returned to it, DUH.  Don't do this.
	.continue:
		
		; Is this a null string
		mov byte al, [ebx]
		cmp al, 0
		jz .end
		
		; No, start blitting until we hit a null terminator
		mov byte al, [ebx]
		mov byte [ecx], al
		inc ebx
		mov byte al, [ebx]
		cmp al, 0
		jz .end
		add ecx, 2
		jmp .continue
		
	.end:
		pop eax		; Restore eax
		ret

;; End of putmsg32

times 510-($-$$)	db 0
			dw 0xaa55	; Boot sig