;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; entry.asm - 32 Bit entry point for OS code
;; (c) 2001 Breckin Loggins
;; Copy Freely
;;

;; Alright kids, if you've been following along on the code so far you know
;; that right now we're at 0x10000 linear, we're in protected mode, and we
;; just JMPed from the bootsector

;; This kernel is a "technology test", we'll be adding things in as we want
;; to learn about them.  No attempt is made to be conservative with size or
;; aggressive with speed
;; 
;; Here's the memory map as it will look:

; 0x0000 - 0x0fff GDT (this gives 512 GDT Entries, fine for now)
; 0x1000 - 0x17ff IDT (gives us our max 256 IDT Entries)
; 0x1800 - 0xffff ~60K of Free Space (we'll use if for something later, or not)
;	       
; 0x10000 - 0x1ffff Kernel Code (all this stuff) 65K for that for now
; 0x20000 - 0x2ffff Stack Space (65K for it as well)
; 0x30000 - 0x3ffff Task State Segments (Enough for 614 tasks)
; 0x40000 - 0xb7fff Reserved for future use (IOW, will be filled out later)
; 0xb8000 - 0xb9fff Video Screen (80*25*2 Bytes)
; 0xba000 - 0xfffff Not Used right now
;
; 0x100000 - 0x700000 7MB of user space

;; Yes, space is wasted like crazy, but it's all for the sake of simplicity.
;; In the future, TSSs will be allocated in the first 103 (or so) bytes
;; of a processes memory and mapped into it.  Also, you'll noticed tables for 
;; memory management conspicuously missing. Well that was intentional, what
;; I'm going to do is separate memory into 4K "pages" (so they will be 
;; compatible with paging hardware when I turn it on) and 12 bytes out of 
;; each page will be used for memory management. 4 bytes for the back link, 
;; for bytes for the next link, and 4 bytes for any info we need to store.
;; I'm also assuming the user only has 8MB of ram, and only 7 of it will be
;; available for user programs.  When I emplement ram checking in the near 
;; future this will go away.  Also has I have to start mapping I/O addresses 
;; and such from the hardware a lot of stuff in the <1MB region will be moved.
;; The morals: 1) When you're the operating system you can do with memory
;; what you darn well like 2) Make things simple at first, just so you can 
;; figure out what the heck is happening.

BITS 32			       ; 32-Bit development
			
; Exported System Calls
GLOBAL _printReg
GLOBAL _start32
GLOBAL _delay

GLOBAL krnltask

; Imported System Calls
EXTERN _loadKernelTSS
EXTERN _newTask
EXTERN _schedAdd

EXTERN _startSound
EXTERN _stopSound
EXTERN _beep

EXTERN _setPITFreq
EXTERN _delayMS

EXTERN lasttask

EXTERN _killFloppyMotor

EXTERN _kbdTest
EXTERN _kbdWaitKeyPress
EXTERN _kbdGetChar
EXTERN _kbdPeekChar

EXTERN _vidClearScreen
EXTERN _vidWrite
EXTERN _vidGotoXY
EXTERN _vidWriteString

EXTERN _initInterrupts

_start32:
	
	call _killFloppyMotor		; It gets annoying otherwise
	
	call _initInterrupts		; Install interrupt handlers
	call _loadKernelTSS		; Install a first TSS for 
					; Multitasking
	push dword 100			; 10 MS time slices
	call _setPITFreq
		
	
	; Clear the screen
	;call _vidClearScreen
	push dword welcomestring
	call _vidWriteString		; Display welcome message
	
 	; Enable NMI
	in ax, 0x70
	and ax, 0x7f
	out 0x70, ax

;	;; Start a new task
	push dword _task1		; ARG1: Offset of the task's entry pt
	push dword 0x2f000		; ARG2: This task's stack pointer
	call _newTask			; Create the task


;	;; Start another new task
	push dword _task2
	push dword 0x2a000
	call _newTask

	;; Start a third task
	push dword _task3
	push dword 0x2b000
	call _newTask

sti					; The kernel code never returns
					; so we must turn on the timer
					; AFTER we add all the tasks we
					; want


krnltask:			; This is defined as where we jump to 
				; when multitasking starts
				
sti
krnlspin: 
	;hlt			; Wait for next interrupt
	jmp krnlspin		; Used now to end the kernel task, 
				; Used in the future as a safety mechanism


;; Sample tasks
_task1:
	inc dword [task1ctr]
	push dword 20
	push dword 1
	call _vidGotoXY
	
	push dword task1msg
	call _vidWriteString
	sti
.spin:
	
	inc dword [task1ctr]
	push dword [task1ctr]
	push dword 1
	call _printReg
	jmp .spin

_task2:
	inc dword [task2ctr]
	push dword 20
	push dword 2
	call _vidGotoXY
	
	push dword task2msg
	call _vidWriteString
	sti
.spin:
	inc dword [task2ctr]
	push dword [task2ctr]
	push dword 2
	call _printReg
	push dword 20
	call _delayMS
	jmp .spin

_task3:
	inc dword [task3ctr]
	push dword 20
	push dword 3
	call _vidGotoXY
	
	push dword task3msg
	call _vidWriteString
	sti
.spin:
	inc dword [task3ctr]
	push dword [task3ctr]
	push dword 3
	call _printReg
	push dword 500
	call _delayMS
	jmp .spin
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Internal Kernel Calls - DON'T USE 
;; OUTSIDE THE KERNEL

;; FUNCTION: _delay
;; ARGS: none
;; PURPOSE: The calling overhead, etc.. should be enough delay for all I/O

_delay:
	jmp .next
.next:
	ret
	
;; FUNCTION: _printReg
;; ARGUMENTS: ARG1 32-Bits the value to print
;;	      ARG2 8-Bits the row to print (0-24)
;; PURPOSE: Prints the 32 bit value of a register starting at 0xb8014

_printReg:
	push eax
	push ebx
	push ecx
	push edx
	
	;; Setup
	; Calculate the row
	mov eax, 160			; 80 Col * 2 Bytes
	xor ebx, ebx	
	mov bl, [esp+20]		; Row offset
	mul ebx
	add eax, 0xb8000		; Offset into framebuf
	mov edx, eax			; Save it
	
	mov eax, [esp+24]		; Character
	
	mov ecx, 0x10			; 8 Digits, 8 attributes

.continue
	mov ebx, eax
	and bl, 0x0F
	cmp bl, 0x9
	ja .hex
	add bl, 0x30
	jmp .print
	
.hex:
	add bl, 0x37

.print:	
	mov byte [edx + 14 + ecx], bl
	mov byte [edx + 15 + ecx], 0x07
	sub ecx, 2
	shr eax, 4
	
	cmp ecx, 0
	jnz .continue
	
	pop edx
	pop ecx
	pop ebx
	pop eax
	ret 8
; End of _printReg


;; Data Area

task1ctr
	dd 0
task2ctr
	dd 0

task3ctr
	dd 0

; The IDT Register, 6 Bytes
IDTR
	dw 0x90		 ; Size (18 entries times 8 Bytes/entry)
	dd 0x1000	 ; Base 0x1000 per our memory map

welcomestring
	db 'Welcome to SimpleOS 0.0.0b', 0

task1msg
	db '<-- This is Task1, no delay', 0
	
task2msg
	db '<-- This is Task2, 20ms delay', 0

task3msg
	db '<-- This is Task3, 500ms delay', 0
