;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; KBD.ASM - Keyboard Driver for SimpleOS
;; (c) 2001 Breckin Loggins
;; Distributed under the GNU General Public License
;;

;; Function Exports

GLOBAL _kbdIrqHandler			; Self-explanatory
GLOBAL _kbdTest
GLOBAL _kbdGetChar			; Gets a character out of the buffer
GLOBAL _kbdWaitKeyPress			; Spins until the status bit is 1
GLOBAL _kbdPeekChar			; Gets the next char out of the buffer
					; but without blocking or decrementing
					; the buffer

;; Function imports
EXTERN _printReg

;; Public Functions

;; FUNCTION: _kbdTest
;; ARGS: none
;; PURPOSE: Temp function for testing functionality

_kbdTest:
	
ret
; End _kbdTest

;; FUNCTION: _kbdWaitKeyPress	
;; ARGS:     none
;; PURPOSE:  blocks until there's a keypress
_kbdWaitKeyPress:
	push eax

.cont:
	mov al, [kbdstatus]
	and al, 1
	jz .cont
	
	; Reset the status bit
	mov al, [kbdstatus]
	and al, 0xfe	; Zero out the first bit
	mov [kbdstatus], al
	
	pop eax
	ret

;; FUNCTION: _kbdGetChar
;; ARGS:	none
;; PURPOSE:  Gets the next char out of the buffer, this call blocks, which
;;	     means that if there are no chars in the buffer, it'll sit here 
;;	     until there is one.  So for your sake leave interrupts enabled if
;;	     you call this function in kernel mode!
;; RETURNS:  The scan code is in eax
_kbdGetChar:
	push ebx
	xor eax, eax
	
.wait:
	; Are we already at the beginning of the keyboard buffer?
	mov ebx, [bufferptr]
	cmp ebx, kbdbuffer
	jle .wait		; Just sit here and wait until something
				; shows up
	
	
	; Somethings here, dec the bufferptr and get the char
	dec dword [bufferptr]	
	
	
	mov ebx, [bufferptr]	; Get the pointer
	mov al, [ebx]		; Get what's there
		
	pop ebx
	ret
; End _kbdGetChar

;; FUNCTION: _kbdPeekChar	
;; ARGS: none
;; PURPOSE: Peeks in the keyboard buffer and gets the last key pressed, does
;;	    not decrement the buffer and does not block
_kbdPeekChar:
	push ebx
	
	xor eax, eax
	
	mov ebx, [bufferptr]	; Get the pointer
	mov al, [ebx]		; Get what's there
		
	pop ebx
	ret
; End _kbdPeekChar


;; FUNCTION: _kbdIrqHandler
;; ARGS: none
;; PURPOSE:  Called whenever a keyboard interrupt is triggered

_kbdIrqHandler:
	
	push eax
	push ebx
	push ecx
	
	; Display the scancode
	xor eax, eax
	in al, 0x60
	
	; We can use eax again
	; Test for scancodes, send them where they need to go, this sucks
	mov ebx, [bufferptr]	; Get a pointer to the buffer
	
	cmp al, 1		; ESC
	je .jend			
	jmp .next1
.jend:
	jmp .end

.next1:
	cmp al, 2		
	je .jsc2
	jmp .next2
.jsc2:
	jmp .sc2

.next2:
	cmp al, 3
	je .jsc3
	jmp .next3
.jsc3:
	jmp .sc3

.next3:
	cmp al, 4
	je .jsc4
	jmp .next4
.jsc4:
	jmp .sc4
	
.next4:
	cmp al, 5
	je .jsc5
	jmp .next5
.jsc5:
	jmp .sc5
	
.next5:
	cmp al, 6
	je .jsc6
	jmp .next6
.jsc6:
	jmp .sc6
	
.next6:
	cmp al, 7
	je .jsc7
	jmp .next7
.jsc7:
	jmp .sc7
	
.next7:
	cmp al, 8
	je .jsc8
	jmp .next8
.jsc8:
	jmp .sc8
	
.next8:
	cmp al, 9
	je .jsc9
	jmp .next9
.jsc9:
	jmp .sc9
	
.next9:
	cmp al, 10
	je .jsc10
	jmp .next10
.jsc10:
	jmp .sc10
	
.next10:
	cmp al, 11
	je .jsc11
	jmp .next11
.jsc11:
	jmp .sc11

.next11:
	cmp al, 12
	je .jsc12
	jmp .next12
.jsc12:
	jmp .sc12
	
.next12:
	cmp al, 13
	je .jsc13
	jmp .next13
.jsc13:
	jmp .sc13
	
.next13:
	cmp al, 14
	je .jsc14
	jmp .next14
.jsc14:
	jmp .sc14
	
.next14:
	cmp al, 15
	je .jsc15
	jmp .next15
.jsc15:
	jmp .sc15
	
.next15:
	cmp al, 16
	je .jsc16
	jmp .next16
.jsc16:
	jmp .sc16
	
.next16:
	cmp al, 17
	je .jsc17
	jmp .next17
.jsc17:
	jmp .sc17
	
.next17:
	cmp al, 18
	je .jsc18
	jmp .next18
.jsc18:
	jmp .sc18
	
.next18:
	cmp al, 19
	je .jsc19
	jmp .next19
.jsc19:
	jmp .sc19
	
.next19:
	cmp al, 20
	je .jsc20
	jmp .next20
.jsc20:
	jmp .sc20
	
.next20:
	cmp al, 21
	je .jsc21
	jmp .next21
.jsc21:
	jmp .sc21
	
.next21:
	cmp al, 22
	je .jsc22
	jmp .next22
.jsc22:
	jmp .sc22
	
.next22:
	cmp al, 23
	je .jsc23
	jmp .next23
.jsc23:
	jmp .sc23
	
.next23:
	cmp al, 24
	je .jsc24
	jmp .next24
.jsc24:
	jmp .sc24
	
.next24:
	cmp al, 25
	je .jsc25
	jmp .next25
.jsc25:
	jmp .sc25
	
.next25:
	cmp al, 26
	je .jsc26
	jmp .next26
.jsc26:
	jmp .sc26
	
	
.next26:
	cmp al, 27
	je .jsc27
	jmp .next27
.jsc27:
	jmp .sc27
	
.next27:
	cmp al, 28
	je .jsc28
	jmp .next28
.jsc28:
	jmp .sc28
	
.next28:
	cmp al, 30
	je .jsc30
	jmp .next29
.jsc30:
	jmp .sc30
	
.next29:
	cmp al, 31
	je .jsc31
	jmp .next30
.jsc31:
	jmp .sc31
	
.next30:
	cmp al, 32
	je .jsc32
	jmp .next31
.jsc32:
	jmp .sc32
	
.next31:
	cmp al, 33
	je .jsc33
	jmp .next32
.jsc33:
	jmp .sc33
	
.next32:
	cmp al, 34
	je .jsc34
	jmp .next33
.jsc34:
	jmp .sc34
	
.next33:
	cmp al, 35
	je .jsc35
	jmp .next34
.jsc35:
	jmp .sc35
	
.next34:
	cmp al, 36
	je .jsc36
	jmp .next35
.jsc36:
	jmp .sc36
	
.next35:
	cmp al, 37
	je .jsc37
	jmp .next36
.jsc37:
	jmp .sc37
	
.next36:
	cmp al, 38
	je .jsc38
	jmp .next37
.jsc38:
	jmp .sc38
	
.next37:
	cmp al, 39
	je .jsc39
	jmp .next38
.jsc39:
	jmp .sc39
	
.next38:
	cmp al, 40
	je .jsc40
	jmp .next39
.jsc40:
	jmp .sc40
	
.next39:
	cmp al, 41
	je .jsc41
	jmp .next40
.jsc41:
	jmp .sc41
	
.next40:
	cmp al, 44
	je .jsc44
	jmp .next41
.jsc44:
	jmp .sc44
	
.next41:
	cmp al, 45
	je .jsc45
	jmp .next42
.jsc45:
	jmp .sc45
	
.next42:
	cmp al, 46
	je .jsc46
	jmp .next43
.jsc46:
	jmp .sc46
	
.next43:
	cmp al, 47
	je .jsc47
	jmp .next44
.jsc47:
	jmp .sc47
	
.next44:
	cmp al, 48
	je .jsc48
	jmp .next45
.jsc48:
	jmp .sc48
	
.next45:
	cmp al, 49
	je .jsc49
	jmp .next46
.jsc49:
	jmp .sc49
	
.next46:	
	cmp al, 50
	je .jsc50
	jmp .next47
.jsc50:
	jmp .sc50
	
.next47:
	cmp al, 51
	je .jsc51
	jmp .next48
.jsc51:
	jmp .sc51
	
.next48:
	cmp al, 52
	je .jsc52
	jmp .next49
.jsc52:
	jmp .sc52
	
.next49:
	cmp al, 53
	je .jsc53
	jmp .next50
.jsc53:
	jmp .sc53
	
.next50:
	jmp .end		; We didn't get the scan codes we were looking
				; for
; The scan code routines, yup these suck too :)
.sc2:
	mov ah, 0x31
	jmp .incptr

.sc3:
	mov ah, 0x32
	jmp .incptr
	
.sc4:
	mov ah, 0x33
	jmp .incptr
	
.sc5:
	mov ah, 0x34
	jmp .incptr

.sc6:
	mov ah, 0x35
	jmp .incptr

.sc7:
	mov ah, 0x36
	jmp .incptr
	
.sc8:
	mov ah, 0x37
	jmp .incptr
	
.sc9:
	mov ah, 0x38
	jmp .incptr
	
.sc10:
	mov ah, 0x39
	jmp .incptr

.sc11:
	mov ah, 0x30
	jmp .incptr
	
.sc12:
	mov ah, 0x2d
	jmp .incptr

.sc13:
	mov ah, 0x3d
	jmp .incptr

.sc14:
	mov ah, 0x08		; Backspace?
	jmp .incptr

.sc15:
	mov ah, 0x0b		; Tab??
	jmp .incptr

.sc16:
	mov ah, 0x67		; q
	jmp .incptr

.sc17:
	mov ah, 0x77		; w
	jmp .incptr

.sc18:
	mov ah, 0x65		; e
	jmp .incptr

.sc19:
	mov ah, 0x72		; r
	jmp .incptr
	
.sc20:
	mov ah, 0x74		; t
	jmp .incptr

.sc21:
	mov ah, 0x79		; y
	jmp .incptr

.sc22:
	mov ah, 0x75		; u
	jmp .incptr

.sc23:
	mov ah, 0x69		; i
	jmp .incptr	

.sc24:
	mov ah, 0x6f		; o
	jmp .incptr

.sc25:
	mov ah, 0x70		; p
	jmp .incptr
	
.sc26:
	mov ah, 0x5b		; [
	jmp .incptr
	
.sc27:
	mov ah, 0x5d		; ]
	jmp .incptr
	
.sc28:
	mov ah, 0x0d		; CR
	jmp .incptr
	
; sc29 is ctrl key, don't put it in the buffer

.sc30:
	mov ah, 0x61		; a
	jmp .incptr

.sc31:
	mov ah, 0x73		; s
	jmp .incptr
	
.sc32:
	mov ah, 0x64		; d
	jmp .incptr
	
.sc33:
	mov ah, 0x66		; f
	jmp .incptr

.sc34:
	mov ah, 0x67		; g
	jmp .incptr

.sc35:
	mov ah, 0x68		; h
	jmp .incptr

.sc36:
	mov ah, 0x6a		; j
	jmp .incptr

.sc37:
	mov ah, 0x6b		; k
	jmp .incptr

.sc38:
	mov ah, 0x6c		; l
	jmp .incptr

.sc39:
	mov ah, 0x3b		; ;
	jmp .incptr
	
.sc40:
	mov ah, 0x27		; '
	jmp .incptr

.sc41:
	mov ah, 0x60		; `
	jmp .incptr
	
; sc42 is left shift

.sc44:
	mov ah, 0x7a		; z
	jmp .incptr
	
.sc45:
	mov ah, 0x78		; x
	jmp .incptr
	
.sc46:
	mov ah, 0x63		; c
	jmp .incptr

.sc47:
	mov ah, 0x76		; v
	jmp .incptr

.sc48:
	mov ah, 0x62		; b
	jmp .incptr
	
.sc49:
	mov ah, 0x6e		; n
	jmp .incptr

.sc50:
	mov ah, 0x6d		; m
	jmp .incptr

.sc51:
	mov ah, 0x2c		; ,
	jmp .incptr
	
.sc52:
	mov ah, 0x2e		; .
	jmp .incptr
	
.sc53:
	mov ah, 0x2f		; /
	jmp .incptr
	

; End 
.incptr:
	; AH now contains the character to put in the buffer, if any
	mov cl, [kbdstatus]
	or cl, 1			; Turn the status bit on
	mov [kbdstatus], cl
	
	mov byte [ebx], ah
	inc ebx
	cmp ebx, kbdbuffer + 512	; Are we at the end?
	je .wrap			; Yup
	mov [bufferptr], ebx
	jmp .end

.wrap:
	mov ebx, kbdbuffer		; Wrap around
	mov [bufferptr], ebx
.end:
	pop ecx
	pop ebx
	pop eax
	ret

;; Helper functions

;; FUNCTION: WaitCommand
;; ARGS: none
;; PURPOSE:  Waits for the command buffer to clear so you can send the 
;;		second byte of the command

WaitCommand:

	push eax
	
.continue:
	in al, 0x64		; Read in the port
	and al, 1		; Mask off all but the last bit
	cmp al, 1		; Are we ready yet?
	jne .continue		; Nope, keep going
	
	
	pop eax
	ret
; End WaitCommand

;; FUNCTION: SetLEDs
;; ARGS: Push a byte onto the stack with the status of each led in the 
;;	low order 3 bits
;; PURPOSE: Turns the lights on the keyboard on and off

SetLEDs:

	push eax
	
	xor eax, eax
	mov ah, [esp+8]		; Get the keyboard code
	
	mov al, 0xed		; Set LED Command
	
	out 0x60, al		; Send the command
	call WaitCommand	; Wait for buffer to clear
	mov al, ah		; Get those LEDs
	out 0x60, al		; Send 'em
	
	pop eax
	
	ret 4
; End SetLEDs	

kbdbuffer
	db 0
	times 520 db 0
	
bufferptr
	dd kbdbuffer			; This is where we are in the buffer
	
; STATUS STRUCTURE:
;	bit 0:	0 = no new char in the buffer
;		1 = new char in the buffer
kbdstatus
	db 0				; Status of the keyboard