;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SPEAKER.ASM - PC Speaker driver
;; (c) 2001 Breckin Loggins
;; Copy Freely
;;

; Function Exports
GLOBAL _startSound
GLOBAL _stopSound
GLOBAL _beep

; Function imports
EXTERN _printReg
EXTERN _delayMS

;; FUNCTION: _startSound
;; ARGUMENTS: Push the frequency 
;; PURPOSE: Attaches the pc speaker to the timer, starting a sound

_startSound:
	
	; Load timer 2 with a new countdown value
	push eax
	push ebx
	push edx
		
	; Calculate the countdown value for the frequency
	mov ebx, [esp+16]	; Save the frequency
	mov eax, 0x1234DC	; System Timer frequency
	xor edx, edx
	div ebx	  	; Calculate the countdown
		
	; Tell the timer we're changing the countdown
	mov bx, ax	; Save the countdown
	mov ax, 0xb6
	out 0x43, ax
	
	; Change the countdown
	mov ax, bx
	and ax, 0x0F
	out 0x42, ax
	mov ax, bx
	shr ax, 8
	and ax, 0x0F
	out 0x42, ax
	
	; Attach the timer to the speaker
	in ax, 0x61
	or ax, 3
	out 0x61, ax
	
	; The speaker should now be producing your selected annoying tone
	pop edx
	pop ebx
	pop eax
	ret 4
; End _startSound
	
;; FUNCTION: _stopSound
;; ARGUMENTS: None
;; PURPOSE: Detaches the speaker from the system timer, stopping the sound

_stopSound:

	push eax
	in ax, 0x61
	and ax, 0xfc
	out 0x61, ax
	
	pop eax
	ret
; End _stopSound

;; FUNCTION: _beep
;; ARGUMENTS: Push the amount of ms to beep onto the stack
;; PURPOSE: Beeps the pc speaker for a certain amount of timer

_beep:
	push eax
	
	mov eax, [esp+8]		; Get the arg
	
	push dword 640			; Play Some note, prob a C 7Oct
	call _startSound
	push eax
	call _delayMS			; Delay for eax ms
	call _stopSound
	
	pop eax
	
	ret 4