;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VIDEO.ASM - Video Support Routines for SimpleOS
;; (c)2001 Breckin Loggins
;; This code is distributed under the terms of the GNU
;; General Public License
;;

;; Function Exports
GLOBAL _vidClearScreen
GLOBAL _vidGotoXY
GLOBAL _vidWrite
GLOBAL _vidWriteString
;GLOBAL _vidCursorOn
;GLOBAL _vidCursorOff

;; Function Imports
EXTERN _printReg
EXTERN _kbdWaitKeyPress

;; Public Functions

;; FUNCTION: _vidClearScreen
;; ARGS:	none
;; PURPOSE:   Clears the screen and returns the cursor to 0, 0

_vidClearScreen:
	push eax
	push ebx
	push ecx
	push edi
	
	xor eax, eax
	xor ebx, ebx
	
	; Find out how many words to mov
	mov al, [maxx]
	mov bl, [maxy]
	mul ebx
	mov ecx, eax		; This is how many words
	
	; Enter the defalt attribute and character
	mov ax, 0x0700
	
	; Enter the destination offset
	mov edi, [vidbuffer]
	
	cld
	rep stosw	; Do It!
	
	; Reset the cursor position
	mov byte [curx], 0
	mov byte [cury], 0
	
	; We're done	
	pop edi
	pop ecx
	pop ebx
	pop eax
	ret
; End _vidClearScreen

;; FUNCTION: _vidGotoXY
;; ARGS:	push x, push y
;; PURPOSE: Goes to the specified screen (invalid values are ignored)
_vidGotoXY:
	push eax
	
	xor eax, eax
	
	mov al, [esp+12]	; AL is X
	mov ah, [esp+8]		; AH is Y
	
	cmp al, [maxx]
	ja .ignore
	cmp ah, [maxy]
	ja .ignore
	
	mov [curx], al
	mov [cury], ah
	
	
.ignore:
	pop eax
	ret 8
; End _vidGotoXY

;; FUNCTION: _vidWrite
;; ARGS: push one character plus attribute on the stack
;; PURPOSE: Writes a character + attrib to curx, cury
_vidWrite:
	push eax
	push ebx
	
	mov bx, [esp+12]	; Now we have char and attrib
	
	push dword [curx]
	push dword [cury]
	call XYToOffset
	
	; EAX now has the offset
	mov byte [eax], bh
	mov byte [eax+1], bl
	
	pop ebx
	pop eax
	ret 4
; End _vidWrite

;; FUNCTION: _vidWriteString
;; ARGS: push the offset of a null terminated string on to the stack
;; PURPOSE: Writes the string specified to curx, cury
_vidWriteString:
	push eax
	push ebx
	push esi
	
	mov esi, [esp+16]		; Get the offset of the string
	push dword [curx]
	push dword [cury]
	call XYToOffset			; XY Now contains the starting offset
	
.continue:
	xor ebx, ebx			; Zero out the temp storage area
	mov bl, [esi] 
	cmp bl, 0			; Zero?
	jz .end
	
	; No, continue
	xor ebx, ebx
	mov bl, 0x07
	mov bh, [esi]			; Get the char
	push ebx			
	call _vidWrite			; Put the character there
	inc esi				; Plus one
	inc byte [curx]
	jmp .continue

.end:
	
	pop esi
	pop ebx
	pop eax
	ret 4
; End _vidWriteString

;; Helper Functions

;; FUNCTION: XYToOffset 
;; ARGS:     Push x, Push y (dwords please)
;; PURPOSE: Converts the x, y coords given into an offset into the framebuffer
;; RETURNS: eax contains the offset

XYToOffset:
	push ebx
	push ecx
	push edx
	
	xor eax, eax
	xor ecx, ecx
	xor edx, edx
	
	mov cl, [esp+20]	; CL contains X
	mov ch, [esp+16]	; CH contains Y
	
	; Algorithm is: vidbuffer + ((maxx * y) + x ) * 2)
	mov al, [maxx]
	mov dl, ch
	mul edx
	mov ch, 0
	add eax, ecx
	mov ebx, 2
	mul ebx
	
	add eax, [vidbuffer]
	
	pop edx
	pop ecx
	pop ebx
	ret 8

;; Data Area

vidbuffer	dd	0xb8000		; Offset of screen buffer
curx		db	0x0		; Current x pos
cury		db	0x0		; Current y pos
maxx		db      80		; Default 80 columns
maxy		db	25		; Default 25 rows


